#!/bin/sh

# This helper script will handle deploying a athena workgroup
set -e

source /scripts/utils.sh
source /scripts/terraform.sh

check_access ${appsNamespace} configmaps

tfAdditionalVarsFile=${tfRoot}/additional_settings.auto.tfvars

echo "iam_policy_prefix = \"${externalNamingPrefix}\"" >> ${tfAdditionalVarsFile}
echo "result_location =  \"${RESULT_LOCATION}\" " >> ${tfAdditionalVarsFile}

terraform_init
terraform_plan
terraform_apply
terraform_output

print_header "Dump output to ConfigMap"
arnConfigMapName=${namingPrefix}-arn
terraform_get arn arnValue
terraform_get athena_workgroup_name athenaWorkgroupName
terraform_get athena_result_output_location athenaResultOutputLocation
terraform_get read_only_policy readOnlyPolicy
terraform_get read_write_policy readWritePolicy

cat >/tmp/config.yaml <<EOF
apiVersion: v1
kind: ConfigMap
metadata:
  name: ${arnConfigMapName}
data:
  ARN: ${arnValue}
  ReadOnlyPolicy: ${readOnlyPolicy}
  ReadWritePolicy: ${readWritePolicy}
EOF

run_cmd cat "/tmp/config.yaml"
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${appsNamespace} apply -f /tmp/config.yaml
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${infraNamespace} apply -f /tmp/config.yaml

echo "Config map ${arnConfigMapName}"
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${appsNamespace} get cm ${arnConfigMapName} -o yaml

cprint "$0 done "
