#!/bin/sh

# Make sure localstack is present, give instructions if not found (hard error) and wire up 
# the cross-namespace access pattern so apps and infra both can access localstack 
source scripts/utils.sh

if [ -z "${appsNamespace}" ] ; then
    echo "Must set env appsNamespace"
    exit 1
fi

if [ -z "${infraNamespace}" ] ; then
    echo "Must set env infraNamespace"
    exit 1
fi

# Check for localstack already present.  Fail hard if not found and point to where to learn how to deploy it.
if ! wait-until -t 5 -i 2 true kubectl --namespace "${infraNamespace}" get deployment "localstack"; then
    echo "ERROR: You must install localstack first!"
    echo "See https://gitlab.bit9.local/mcs2/infrastructure-charts/#localstack for instructions"
    exit 1
fi

# Now create the ExternalName service pointing back to the localstack service
cat > /tmp/externalservice.yaml <<EOF
kind: Service
apiVersion: v1
metadata:
  name: "localstack"
  namespace: ${appsNamespace}
spec:
  type: ExternalName
  externalName: "localstack.${infraNamespace}.svc.cluster.local"
EOF

# Now create the ExternalName configmap pointing back to the localstack service
cat > /tmp/externalhostnameconfig.yaml <<EOF
kind: ConfigMap
apiVersion: v1
metadata:
  name: localstack-endpoint
  namespace: ${appsNamespace}
data:
  endpoint: "localstack.${infraNamespace}.svc.cluster.local"
EOF

# Dump out the YAML we're going to create to help debugging if something goes bad
run_cmd nl /tmp/externalservice.yaml

echo "Creating (or updating) service in ${appsNamespace}"
run_cmd kubectl apply -f /tmp/externalservice.yaml || echo "Failed to create external name service, ignoring"

echo "Creating (or updating) configmap in ${appsNamespace}"
run_cmd kubectl apply -f /tmp/externalhostnameconfig.yaml || echo "Failed to create configmap, ignoring"

set -e
echo "Wail for localstack to be able to list s3 buckets ( timeout 30 sec , interval 5 secs )...."
wait-until -t 30 -i 5 true aws --endpoint-url=http://localstack:4566 s3 ls 

echo "$(date) done"
