################################################
# Helper Functions
################################################
cprint() {
    nc='\033[0m' # No Color
    case $2 in
	'red')
	    color='\033[0;31m'
	    ;;
	'green')
	    color='\033[0;32m'
	    ;;
	'yellow')
	    color='\033[0;33m'
	    ;;

	'blue')
	    color='\033[0;34m'
	    ;;
	'purple')
	    color='\033[0;35m'
	    ;;
	'cyan')
	    color='\033[0;36m'
	    ;;

	*)
	    color='\033[0;34m'
	    #color=$nc
	    ;;
    esac
    printf "${color}$1${nc}\n"
}

error_exit() {
  msg="${1}"
  cprint "ERROR: $msg" 'red'
  exit 1 
}

print_header() {
    line="============================================================"
    text=$1
    cprint "$(printf "%s %s\n" "${text}" "${line:${#text}}")" "purple"
}

check_access() {
    namespace=$1
    type=$2
    echo "Verifying ${type} can be created in ${namespace}: $(kubectl --namespace ${namespace} auth can-i create ${type})"
}

create_secret_password() {
    secretName=$1
    namespace=$2
    password=$3
    if wait-until -t 30 -i 5 true kubectl --namespace "${namespace}" get secret "${secretName}"; then
      echo "Secret ${secretName} already exists."
    else
      kubectl --namespace "${namespace}" create secret generic "${secretName}" --from-literal=password="${password}"
    fi
}

create_secret_user() {
    secretName=$1
    namespace=$2
    user=$3
    if wait-until -t 30 -i 5 true kubectl --namespace "${namespace}" get secret "${secretName}"; then
      echo "Secret ${secretName} already exists."
    else
      kubectl --namespace "${namespace}" create secret generic "${secretName}" --from-literal=user="${user}"
    fi
}

get_secret_cred() {
    secretName=$1
    namespace=$2
    key=$3
    secret_cred=$(kubectl -n "${namespace}"  get secret "${secretName}" --template={{.data."${key}"}} | base64 -d)
    echo $secret_cred
}

create_external_service() {
    serviceName=$1
    namespace=$2
    endpoint=$3
    if wait-until -t 30 -i 5 true kubectl --namespace "${namespace}" get service ${serviceName}; then
      echo "Service ${serviceName} already exists."
    else
      run_cmd kubectl --namespace=${namespace} create service externalname ${serviceName} --external-name ${endpoint}
    fi
}

create_hostname_configmap() {
    cmName=$1
    namespace=$2
    shift 2
    fromLiteral=$@
    if wait-until -t 30 -i 5 true kubectl --namespace "${namespace}" get configmap ${cmName}; then
      echo "Configmap ${cmName} already exists."
    else
      run_cmd kubectl --namespace=${namespace} create configmap ${cmName} $fromLiteral
    fi
}

gen_password() {
    openssl rand -hex 32
}

# Params: (1) Prefix Env Search String (2) Terraform Varibale Name (2) tfvars Filename
append_iam_roles_from_env_to_tf_vars() {
    searchString=$1
    tfvarName=$2
    tfvarFile=$3

    cprint "Searching ENV for ${searchString} IAM Roles."

    printenv | grep "${searchString}" | (while read -r line; do
    echo "Processing ${searchString} in env: $line"
    IFS="=" read -ra splitLine <<< "${line}"
    roleName=${splitLine[1]}
    export iam_roles="${iam_roles} \"${roleName}\","
    done

    echo "${tfvarName} = [${iam_roles::-1}]" >> ${tfvarFile})
}

# Params: (1) Prefix Env Search String (2) Terraform Variable Name (2) tfvars Filename, produces a Map{}
terraform_extract_map_from_env_to_tf_vars() {
  unset opt OPTARG OPTIND
  while getopts s:v:f: flag
  do
    case "${flag}" in
      s) searchString=${OPTARG};;
      v) tfvarName=${OPTARG};;
      f) tfvarFile=${OPTARG};;
      *) ;;
    esac
  done
  echo "searchString: $searchString";
  echo "tfvarName: $tfvarName";
  echo "tfvarFile: $tfvarFile";

  tmp_file="${tfRoot}/${tfvarName}.txt"
  touch "${tmp_file}"

  # Note: parentheses around the while loop are required as while loops in shell run in a sub-process
  printenv | grep "$searchString" | (while read -r line ; do
    printf "Processing: %s\n" "$line"
    IFS="=" read -ra splitLine <<< "${line}"
    key=${splitLine[0]:${#searchString}}
    value=${splitLine[1]}
    echo "Found: \"${key}\"=\"${value}\""
    printf "%s=\"%s\"\n" "${key}" "${value}"  >> "${tmp_file}"
  done
  echo "$tfvarName={$(cat "$tmp_file")}" >> "${tfvarFile}" )

  printf "Wrote var.%s to %s\n" "$tfvarName" "${tfvarFile}"
  rm "${tmp_file}" 2>/dev/null
}

# Params: (1) Prefix Env Search String (2) Terraform Variable Name (2) tfvars Filename, produces a Set[]
terraform_extract_set_from_env_to_tf_vars() {
  unset opt OPTARG OPTIND
  while getopts s:v:f: flag
  do
    case "${flag}" in
      s) searchString=${OPTARG};;
      v) tfvarName=${OPTARG};;
      f) tfvarFile=${OPTARG};;
      *) ;;
    esac
  done
  echo "searchString: $searchString";
  echo "tfvarName: $tfvarName";
  echo "tfvarFile: $tfvarFile";

  tmp_file="${tfRoot}/${tfvarName}.txt"
  touch "${tmp_file}"

  # Note: parentheses around the while loop are required as while loops in shell run in a sub-process
  printenv | grep "$searchString" | (while read -r line ; do
    printf "Processing: %s\n" "$line"
    IFS="=" read -ra splitLine <<< "${line}"
    key=${splitLine[0]:${#searchString}}
    value=${splitLine[1]}
    printf "\"%s\",\n" "${value}" >> "${tmp_file}"
  done
  echo "$tfvarName=[$(cat "$tmp_file")]" >> "${tfvarFile}" )

  printf "Wrote var.%s to %s\n" "$tfvarName" "${tfvarFile}"
  rm "${tmp_file}" 2>/dev/null
}

# Print Header for everything that sources this..
print_header "Begin of $0 from ${HOSTNAME}"
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"
if [[ ! "${PATH}x" =~ ${DIR} ]]; then
    export PATH=${PATH}:${DIR}
fi